#!/usr/bin/perl -w

use strict;

#################################### NOTES ######################################
#Written by: Elizabeth Page-Gould,lpgould@wjh.harvard.edu                       #
#First used in: Page-Gould, Mendoza-Denton, Alegre, & Siy (under review)        #
#Purpose: Randomly assigns participants to describe one of two friends, and     #
#         generates necessary experiment files to make one friend accessible    #
#         (a friendship prime for each participant and a condition log for all  #
#         participants.  It needs an input file that has id numbers and the     #
#         names and group memberships of two different types of friends. The    #
#         script also needs the prime template file ('friendshipPrime.tmpl') in #
#         the same directory as the script.                                     #
#Dependencies:-1-You must install the following program on your Unix system:    #
#               1.1_perl 5.8 (type `sudo apt-get install perl perl-doc`)        #
#               1.2_To actually collect data from the prime's input fields, you #
#                   will need MediaLab v.2006 or higher, or your own web server #
#                   (see Usage Point 6.1 below)                                 #
#Usage:  To generate a group of friendship primes:                              #
#        1-copy this script into a directory with the input participantInfo.csv #
#          file (see the example input file, 'participantInfo.csv')             #
#        2-make the script to be executable                                     #
#          _2.1_type `chmod 755 generateFriendshipPrimes.pl` on the command line#
#        3-make sure the template file 'friendshipPrime.tmpl' is also in the    #
#          same directory as this script, and that there is a subdirectory      #
#          called "primes."  If these are missing, try downloading the          #
#          friendshipPrimes package again from http://rascl.berkeley.edu/tools  #
#        4-invoke the script                                                    #
#        5-the friendship primes and a condition log will be created in the     #
#          "primes" subdirectory                                                #
#        6-Two things of note:                                                  #
#          6.1_We integrated these primes with MediaLab v.2006, and so the data #
#              was collected automatically from the html <input> and <textbox>  #
#              fields by MediaLab since we used a form tag: <form method=POST>. #
#              HOWEVER, if you are going to use/process these primes through the#
#              use of a webserver, then you will need to add an 'action=...'    #
#              attribute to the <form ...> tag or you won't be able to collect  #
#              any data! (e.g., '<form method=post action='processSurvey.pl'>') #
#          6.2_In the 'primes' subdirectory, there is a 'friendshipPrime.css'   #
#              file that gives the surveys their color scheme, font attributes, #
#              etc.  You can edit this file however you like to make the primes #
#              match your survey's overall color scheme and style. There is also#
#              an 'images' subdirectory which must be kept in the same directory#
#              with the friendship primes so that the Aron,Aron,&Smollan(1992)  #
#              IOS circles can be displayed in the prime                        #
#################################################################################

generateFriendshipPrimes();

sub generateFriendshipPrimes {makeFiles(getParticipantInfo()) && print "All done! :)\n";}

sub getParticipantInfo
{
    open(IN,'participantInfo.csv') or die $!, "Couldn\'t read participant info input file.\n";
    my @datalines = <IN>;
    close(IN);
    my %data;
    foreach my $line (@datalines)
    {
        $line =~ s/[\r|\n]//si;
        my @linedata = split(/\,/,$line);
        my $id = $linedata[0];
        $data{$id} = \@linedata;
    }
    return(\%data);
}

sub makeFiles {generateHTML(generateAndRecordCondition($_[0]));}

sub generateAndRecordCondition
{
     my %data = %{$_[0]};
     my @ids = sort(keys(%data));
     my (%conditionHash);
     open(LOG,">primes/condition.log") or die $!,"Couldn\'t open condition file for writing\n";
     foreach my $id (@ids)
     {
         my @linedata = @{$data{$id}};
         my $id = $linedata[0];
         my $conditionSeed = int(rand(100));
         my ($condition,$conditionName,$friendsEthnicity);
         if ($conditionSeed < 50) {$condition = 0; $conditionName='Same-Ethnicity'; $friendsEthnicity = $linedata[2];}
         elsif ($conditionSeed >= 50) {$condition = 1; $conditionName='Cross-Ethnicity'; $friendsEthnicity = $linedata[4];}
         else {print "There is something wrong with condition assignment for participant $id\n";}
         $conditionHash{"$id"} = $condition;
         print LOG "$id\,$condition\,$conditionName\,$friendsEthnicity\n";
     }
     close(LOG);
     return(\%data,\%conditionHash);
}

sub generateHTML
{
    my %data = %{$_[0]};
    my %conditionHash = %{$_[1]};
    my @ids = sort(keys(%data));
    my $eligibleNum = $#ids + 1;
    my $sameEthnicityCondition = 0;
    my $crossEthnicityCondition = 0;
    if (-e "friendshipPrime.tmpl")
    {
         print "Beginning to generate HTML files for $eligibleNum participants ...\n";
         foreach my $id (@ids)
         {
             my @linedata = @{$data{$id}};
             my $condition = $conditionHash{"$id"};
             my $friendsName;
             if ($condition == 0) {$friendsName = $linedata[1];$sameEthnicityCondition++;}
             elsif ($condition == 1) {$friendsName = $linedata[3];$crossEthnicityCondition++;}
             else {print "\n\nIt seems like no condition was assigned for ${id}\n";}
             open(TEMPLATE,"friendshipPrime.tmpl") or die $!, "Couldn\'t open template file for reading\n";
             my @friendshipPrimeTemplate = <TEMPLATE>;
             close(TEMPLATE);
             open(OUT,">primes/${id}.html") or die $!, "Couldn\'t open the html file for writing ${id}!\n";
             foreach my $line (@friendshipPrimeTemplate)
             {
                 $line =~ s/<TMPL_VAR NAME=friendName>/${friendsName}/gi;
                 print OUT "$line";
             }
             close(OUT);
             print "\t \.\.\. Just finished ${id}\'s html file\, ${id}\.html!\n";
         }
         print "Finished generating $eligibleNum HTML files for friendship salience primes!\nHad $crossEthnicityCondition in the cross-ethnicity condition and $sameEthnicityCondition in the same-ethnicity condition.\n";
    }
    else {print "Oops!  The friendshipPrime.tmpl was not found in this directory.  Please copy it into this directory and the rerun this script.\n";}
}